@@ -28,41 +28,64 @@ import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.reset;
 import static org.mockito.Mockito.verify;
-import static org.mockito.Mockito.verifyZeroInteractions;
+import static org.mockito.Mockito.verifyNoInteractions;
 import static org.mockito.Mockito.when;
 
 import java.time.Duration;
-import java.time.Instant;
 import java.util.Collections;
+import java.util.Map;
 import java.util.Set;
 
+import io.undertow.UndertowOptions;
+import io.undertow.connector.ByteBufferPool;
+import io.undertow.server.HttpHandler;
 import io.undertow.server.HttpServerExchange;
+import io.undertow.server.protocol.http.HttpServerConnection;
 import io.undertow.server.session.SessionConfig;
 import io.undertow.server.session.SessionListener;
 import io.undertow.server.session.SessionListeners;
+import io.undertow.util.Protocols;
 
+import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 import org.wildfly.clustering.ee.Batch;
 import org.wildfly.clustering.ee.Batcher;
 import org.wildfly.clustering.web.session.ImmutableSession;
-import org.wildfly.clustering.web.session.ImmutableSessionAttributes;
 import org.wildfly.clustering.web.session.ImmutableSessionMetaData;
 import org.wildfly.clustering.web.session.Session;
 import org.wildfly.clustering.web.session.SessionManager;
+import org.wildfly.clustering.web.session.SessionMetaData;
+import org.xnio.OptionMap;
+import org.xnio.StreamConnection;
+import org.xnio.channels.Configurable;
+import org.xnio.conduits.ConduitStreamSinkChannel;
+import org.xnio.conduits.ConduitStreamSourceChannel;
+import org.xnio.conduits.StreamSinkConduit;
+import org.xnio.conduits.StreamSourceConduit;
 
 public class DistributableSessionManagerTestCase {
     private final String deploymentName = "mydeployment.war";
-    private final SessionManager<LocalSessionContext, Batch> manager = mock(SessionManager.class);
+    private final SessionManager<Map<String, Object>, Batch> manager = mock(SessionManager.class);
     private final SessionListener listener = mock(SessionListener.class);
     private final SessionListeners listeners = new SessionListeners();
     private final RecordableSessionManagerStatistics statistics = mock(RecordableSessionManagerStatistics.class);
 
-    private final DistributableSessionManager adapter = new DistributableSessionManager(this.deploymentName, this.manager, this.listeners, this.statistics);
+    private DistributableSessionManager adapter;
 
     @Before
     public void init() {
+        DistributableSessionManagerConfiguration config = mock(DistributableSessionManagerConfiguration.class);
+
+        when(config.getDeploymentName()).thenReturn(this.deploymentName);
+        when(config.getSessionListeners()).thenReturn(this.listeners);
+        when(config.getSessionManager()).thenReturn(this.manager);
+        when(config.getStatistics()).thenReturn(this.statistics);
+        when(config.isOrphanSessionAllowed()).thenReturn(false);
+
+        this.adapter = new DistributableSessionManager(config);
         this.adapter.registerSessionListener(this.listener);
     }
 
@@ -81,6 +104,8 @@ public class DistributableSessionManagerTestCase {
 
     @Test
     public void stop() {
+        when(this.manager.getStopTimeout()).thenReturn(Duration.ZERO);
+
         this.adapter.stop();
 
         verify(this.manager).stop();
@@ -93,13 +118,39 @@ public class DistributableSessionManagerTestCase {
         verify(this.manager).setDefaultMaxInactiveInterval(Duration.ofSeconds(10L));
     }
 
+    @Test
+    public void createSessionResponseCommitted() {
+        // Ugh - all this, just to get HttpServerExchange.isResponseStarted() to return true
+        Configurable configurable = mock(Configurable.class);
+        StreamSourceConduit sourceConduit = mock(StreamSourceConduit.class);
+        ConduitStreamSourceChannel sourceChannel = new ConduitStreamSourceChannel(configurable, sourceConduit);
+        StreamSinkConduit sinkConduit = mock(StreamSinkConduit.class);
+        ConduitStreamSinkChannel sinkChannel = new ConduitStreamSinkChannel(configurable, sinkConduit);
+        StreamConnection stream = mock(StreamConnection.class);
+
+        when(stream.getSourceChannel()).thenReturn(sourceChannel);
+        when(stream.getSinkChannel()).thenReturn(sinkChannel);
+
+        ByteBufferPool bufferPool = mock(ByteBufferPool.class);
+        HttpHandler handler = mock(HttpHandler.class);
+        HttpServerConnection connection = new HttpServerConnection(stream, bufferPool, handler, OptionMap.create(UndertowOptions.ALWAYS_SET_DATE, false), 0, null);
+        HttpServerExchange exchange = new HttpServerExchange(connection);
+        exchange.setProtocol(Protocols.HTTP_1_1);
+        exchange.getResponseChannel();
+
+        SessionConfig config = mock(SessionConfig.class);
+
+        Assert.assertThrows(IllegalStateException.class, () -> this.adapter.createSession(exchange, config));
+    }
+
     @Test
     public void createSessionNoSessionId() {
         HttpServerExchange exchange = new HttpServerExchange(null);
         Batcher<Batch> batcher = mock(Batcher.class);
         Batch batch = mock(Batch.class);
         SessionConfig config = mock(SessionConfig.class);
-        Session<LocalSessionContext> session = mock(Session.class);
+        Session<Map<String, Object>> session = mock(Session.class);
+        SessionMetaData metaData = mock(SessionMetaData.class);
         String sessionId = "session";
 
         when(this.manager.createIdentifier()).thenReturn(sessionId);
@@ -107,6 +158,8 @@ public class DistributableSessionManagerTestCase {
         when(this.manager.getBatcher()).thenReturn(batcher);
         when(batcher.createBatch()).thenReturn(batch);
         when(session.getId()).thenReturn(sessionId);
+        when(session.getMetaData()).thenReturn(metaData);
+        when(metaData.isNew()).thenReturn(true);
 
         io.undertow.server.session.Session sessionAdapter = this.adapter.createSession(exchange, config);
 
@@ -130,7 +183,8 @@ public class DistributableSessionManagerTestCase {
         Batcher<Batch> batcher = mock(Batcher.class);
         Batch batch = mock(Batch.class);
         SessionConfig config = mock(SessionConfig.class);
-        Session<LocalSessionContext> session = mock(Session.class);
+        Session<Map<String, Object>> session = mock(Session.class);
+        SessionMetaData metaData = mock(SessionMetaData.class);
         String sessionId = "session";
 
         when(config.findSessionId(exchange)).thenReturn(sessionId);
@@ -138,6 +192,8 @@ public class DistributableSessionManagerTestCase {
         when(this.manager.getBatcher()).thenReturn(batcher);
         when(batcher.createBatch()).thenReturn(batch);
         when(session.getId()).thenReturn(sessionId);
+        when(session.getMetaData()).thenReturn(metaData);
+        when(metaData.isNew()).thenReturn(true);
 
         io.undertow.server.session.Session sessionAdapter = this.adapter.createSession(exchange, config);
 
@@ -186,7 +242,8 @@ public class DistributableSessionManagerTestCase {
         Batcher<Batch> batcher = mock(Batcher.class);
         Batch batch = mock(Batch.class);
         SessionConfig config = mock(SessionConfig.class);
-        Session<LocalSessionContext> session = mock(Session.class);
+        Session<Map<String, Object>> session = mock(Session.class);
+        SessionMetaData metaData = mock(SessionMetaData.class);
         String sessionId = "session";
 
         when(config.findSessionId(exchange)).thenReturn(sessionId);
@@ -194,12 +251,14 @@ public class DistributableSessionManagerTestCase {
         when(this.manager.getBatcher()).thenReturn(batcher);
         when(batcher.createBatch()).thenReturn(batch);
         when(session.getId()).thenReturn(sessionId);
+        when(session.getMetaData()).thenReturn(metaData);
+        when(metaData.isNew()).thenReturn(false);
 
         io.undertow.server.session.Session sessionAdapter = this.adapter.getSession(exchange, config);
 
         assertNotNull(sessionAdapter);
 
-        verifyZeroInteractions(this.statistics);
+        verifyNoInteractions(this.statistics);
 
         verify(batcher).suspendBatch();
 
@@ -304,58 +363,42 @@ public class DistributableSessionManagerTestCase {
     @Test
     public void getSessionByIdentifier() {
         Batcher<Batch> batcher = mock(Batcher.class);
-        Batch batch = mock(Batch.class);
+        Batch batch1 = mock(Batch.class);
+        Batch batch2 = mock(Batch.class);
         ImmutableSession session = mock(ImmutableSession.class);
-        ImmutableSessionAttributes attributes = mock(ImmutableSessionAttributes.class);
         ImmutableSessionMetaData metaData = mock(ImmutableSessionMetaData.class);
-        String id = "session";
-        String name = "name";
-        Object value = new Object();
-        Set<String> names = Collections.singleton(name);
-        Instant creationTime = Instant.now();
-        Instant lastAccessedTime = Instant.now();
-        Duration maxInactiveInterval = Duration.ofMinutes(30L);
+        String id = "ABC123";
 
         when(this.manager.getBatcher()).thenReturn(batcher);
-        when(this.manager.viewSession(id)).thenReturn(session);
-        when(session.getId()).thenReturn(id);
-        when(session.getAttributes()).thenReturn(attributes);
-        when(attributes.getAttributeNames()).thenReturn(names);
-        when(attributes.getAttribute(name)).thenReturn(value);
+        when(batcher.createBatch()).thenReturn(batch1, batch2);
+        when(this.manager.readSession(id)).thenReturn(session);
         when(session.getMetaData()).thenReturn(metaData);
-        when(metaData.getCreationTime()).thenReturn(creationTime);
-        when(metaData.getLastAccessedTime()).thenReturn(lastAccessedTime);
-        when(metaData.getMaxInactiveInterval()).thenReturn(maxInactiveInterval);
-        when(batcher.createBatch()).thenReturn(batch);
+        when(metaData.isNew()).thenReturn(false);
 
         io.undertow.server.session.Session result = this.adapter.getSession(id);
 
-        assertSame(this.adapter, result.getSessionManager());
-        assertSame(id, result.getId());
-        assertEquals(creationTime.toEpochMilli(), result.getCreationTime());
-        assertEquals(lastAccessedTime.toEpochMilli(), result.getLastAccessedTime());
-        assertEquals(maxInactiveInterval.getSeconds(), result.getMaxInactiveInterval());
-        assertEquals(names, result.getAttributeNames());
-        assertSame(value, result.getAttribute(name));
+        Assert.assertSame(id, result.getId());
 
-        verify(batch).close();
+        verify(batch1).close();
+        verify(batch2).close();
     }
 
     @Test
     public void getSessionByIdentifierNotExists() {
-        String id = "session";
         Batcher<Batch> batcher = mock(Batcher.class);
         Batch batch = mock(Batch.class);
+        String id = "ABC123";
 
         when(this.manager.getBatcher()).thenReturn(batcher);
-        when(this.manager.viewSession(id)).thenReturn(null);
         when(batcher.createBatch()).thenReturn(batch);
+        when(this.manager.readSession(id)).thenReturn(null);
 
         io.undertow.server.session.Session result = this.adapter.getSession(id);
 
-        assertNull(result);
+        Assert.assertNull(result);
 
         verify(batch).close();
+        reset(batch);
     }
 
     @Test
