@@ -23,20 +23,19 @@ package org.jboss.as.ejb3.timerservice;
 
 import java.io.Serializable;
 import java.util.Date;
-import java.util.concurrent.locks.ReentrantLock;
-
+import java.util.concurrent.Semaphore;
 import javax.ejb.EJBException;
 import javax.ejb.ScheduleExpression;
 import javax.ejb.Timer;
 import javax.ejb.TimerHandle;
 
-import org.jboss.as.ejb3.logging.EjbLogger;
 import org.jboss.as.ejb3.component.allowedmethods.AllowedMethodsInformation;
 import org.jboss.as.ejb3.component.allowedmethods.MethodType;
+import org.jboss.as.ejb3.logging.EjbLogger;
 import org.jboss.as.ejb3.timerservice.spi.TimedObjectInvoker;
 
 /**
- * Implementation of EJB3.1 {@link Timer}
+ * Implementation of Enterprise Beans 3.1 {@link Timer}
  *
  * @author <a href="mailto:cdewolf@redhat.com">Carlo de Wolf</a>
  * @version $Revision: $
@@ -45,7 +44,7 @@ public class TimerImpl implements Timer {
     /**
      * The output format with the cached final values, after the first toString() invocation
      */
-    private String toStringTemplate = null;
+    private String toStringTemplate;
     /**
      * Unique id for this timer instance
      */
@@ -77,9 +76,10 @@ public class TimerImpl implements Timer {
     protected final boolean persistent;
 
     /**
-     * A {@link javax.ejb.TimerHandle} for this timer
+     * A {@link javax.ejb.TimerHandle} for this timer, not assigned till
+     * {@code getHandle()} is called.
      */
-    protected final TimerHandleImpl handle;
+    protected TimerHandleImpl handle;
 
     /**
      * The initial (first) expiry date of this timer
@@ -117,7 +117,13 @@ public class TimerImpl implements Timer {
      *
      * todo: we can probably just use a sync block here
      */
-    private final ReentrantLock inUseLock = new ReentrantLock();
+    private final Semaphore inUseLock = new Semaphore(1);
+
+    /**
+     * The executing thread which is processing the timeout task This is only set to executing thread for TimerState.IN_TIMEOUT
+     * and TimerState.RETRY_TIMEOUT
+     */
+    private volatile Thread executingThread;
 
     /**
      * Creates a {@link TimerImpl}
@@ -145,7 +151,6 @@ public class TimerImpl implements Timer {
         this.timerState = builder.timerState;
         this.timerService = service;
         this.timedObjectInvoker = service.getInvoker();
-        this.handle = new TimerHandleImpl(this.id, this.timedObjectInvoker.getTimedObjectId(), service);
     }
 
     /**
@@ -153,7 +158,11 @@ public class TimerImpl implements Timer {
      */
     @Override
     public void cancel() throws IllegalStateException, EJBException {
-        timerService.cancelTimer(this);
+        try {
+            timerService.cancelTimer(this);
+        } catch (InterruptedException e) {
+            throw new EJBException(e);
+        }
     }
 
     /**
@@ -178,32 +187,23 @@ public class TimerImpl implements Timer {
 
     /**
      * {@inheritDoc}
-     *
-     * @see #getTimerHandle()
      */
     @Override
     public TimerHandle getHandle() throws IllegalStateException, EJBException {
         // make sure it's in correct state
         this.assertTimerState();
 
-        // for non-persistent timers throws an exception (mandated by EJB3 spec)
-        if (this.persistent == false) {
-            throw EjbLogger.EJB3_TIMER_LOGGER.invalidTimerHandlersForPersistentTimers("EJB3.1 Spec 18.2.6");
+        // for non-persistent timers throws an exception (mandated by Enterprise Beans 3 spec)
+        if (!persistent) {
+            throw EjbLogger.EJB3_TIMER_LOGGER.invalidTimerHandlersForPersistentTimers("Enterprise Beans 3.1 Spec 18.2.6");
         }
-        return this.handle;
-    }
 
-    /**
-     * This method returns the {@link javax.ejb.TimerHandle} corresponding to this {@link TimerImpl}.
-     * Unlike the {@link #getHandle()} method, this method does <i>not</i> throw an {@link IllegalStateException}
-     * or {@link javax.ejb.NoSuchObjectLocalException} or {@link javax.ejb.EJBException}, for non-persistent timers.
-     * Instead this method returns the {@link javax.ejb.TimerHandle} corresponding to that non-persistent
-     * timer (remember that {@link TimerImpl} creates {@link javax.ejb.TimerHandle} for both persistent and non-persistent timers)
-     *
-     * @return
-     */
-    public TimerHandle getTimerHandle() {
-        return this.handle;
+        synchronized (this) {
+            if (handle == null) {
+                handle = new TimerHandleImpl(id, timedObjectId, timerService);
+            }
+        }
+        return handle;
     }
 
     /**
@@ -216,10 +216,6 @@ public class TimerImpl implements Timer {
         return this.persistent;
     }
 
-    public boolean isTimerPersistent() {
-        return this.persistent;
-    }
-
     /**
      * {@inheritDoc}
      *
@@ -276,7 +272,7 @@ public class TimerImpl implements Timer {
      */
     public void setNextTimeout(Date next) {
         if(next == null) {
-            setTimerState(TimerState.EXPIRED);
+            setTimerState(TimerState.EXPIRED, null);
         }
         this.nextExpiration = next;
     }
@@ -442,6 +438,15 @@ public class TimerImpl implements Timer {
         return this.timerState;
     }
 
+    /**
+     * Returns the executing thread which is processing the timeout task
+     *
+     * @return the executingThread
+     */
+    protected Thread getExecutingThread() {
+        return executingThread;
+    }
+
     /**
      * Asserts that the timer is <i>not</i> in any of the following states:
      * <ul>
@@ -449,24 +454,26 @@ public class TimerImpl implements Timer {
      * <li>{@link TimerState#EXPIRED}</li>
      * </ul>
      *
-     * @throws javax.ejb.NoSuchObjectLocalException
-     *          if the txtimer was canceled or has expired
+     * @throws javax.ejb.NoSuchObjectLocalException if the txtimer was canceled or has expired
      */
     protected void assertTimerState() {
         if (timerState == TimerState.EXPIRED)
-            throw EjbLogger.EJB3_TIMER_LOGGER.timerHasExpired();
+            throw EjbLogger.EJB3_TIMER_LOGGER.timerHasExpired(id);
         if (timerState == TimerState.CANCELED)
-            throw EjbLogger.EJB3_TIMER_LOGGER.timerWasCanceled();
+            throw EjbLogger.EJB3_TIMER_LOGGER.timerWasCanceled(id);
         AllowedMethodsInformation.checkAllowed(MethodType.TIMER_SERVICE_METHOD);
     }
 
     /**
-     * Sets the state of this timer
+     * Sets the state and timer task executing thread of this timer
      *
      * @param state The state of this timer
+     * @param thread The executing thread which is processing the timeout task
      */
-    public void setTimerState(TimerState state) {
+    public void setTimerState(TimerState state, Thread thread) {
+        assert ((state == TimerState.IN_TIMEOUT || state == TimerState.RETRY_TIMEOUT) && thread != null) || thread == null : "Invalid to set timer state " + state + " with executing Thread " + thread;
         this.timerState = state;
+        this.executingThread = thread;
     }
 
     /**
@@ -515,39 +522,30 @@ public class TimerImpl implements Timer {
         return new TimerTask<TimerImpl>(this);
     }
 
-    public void lock() {
-        inUseLock.lock();
+    public void lock() throws InterruptedException {
+        inUseLock.acquire();
     }
 
     public void unlock() {
-        inUseLock.unlock();
+        inUseLock.release();
     }
 
-    /**
-     * A {@link javax.ejb.Timer} is equal to another {@link javax.ejb.Timer} if their
-     * {@link javax.ejb.TimerHandle}s are equal
-     */
     @Override
-    public boolean equals(Object obj) {
-        if (obj == null) {
-            return false;
-        }
-        if (this.handle == null) {
-            return false;
-        }
-        if (obj instanceof TimerImpl == false) {
-            return false;
-        }
-        TimerImpl otherTimer = (TimerImpl) obj;
-        return this.handle.equals(otherTimer.getTimerHandle());
+    public boolean equals(final Object o) {
+        if (this == o) return true;
+        if (o == null || this.getClass() != o.getClass()) return false;
+
+        final TimerImpl otherTimer = (TimerImpl) o;
+
+        if (!id.equals(otherTimer.id)) return false;
+        return timedObjectId.equals(otherTimer.timedObjectId);
     }
 
     @Override
     public int hashCode() {
-        return this.handle.hashCode();
+        return id.hashCode();
     }
 
-
     /**
      * A nice formatted string output for this timer
      * {@inheritDoc}
@@ -567,6 +565,8 @@ public class TimerImpl implements Timer {
             sb.append(this.persistent);
             sb.append(" timerService=");
             sb.append(this.timerService);
+            sb.append(" previousRun=");
+            sb.append(this.previousRun);
             sb.append(" initialExpiration=");
             sb.append(this.initialExpiration);
             sb.append(" intervalDuration(in milli sec)=");
@@ -669,42 +669,6 @@ public class TimerImpl implements Timer {
             return timedObjectId;
         }
 
-        public Date getInitialDate() {
-            return initialDate;
-        }
-
-        public long getRepeatInterval() {
-            return repeatInterval;
-        }
-
-        public Date getNextDate() {
-            return nextDate;
-        }
-
-        public Date getPreviousRun() {
-            return previousRun;
-        }
-
-        public Serializable getInfo() {
-            return info;
-        }
-
-        public Object getPrimaryKey() {
-            return primaryKey;
-        }
-
-        public TimerState getTimerState() {
-            return timerState;
-        }
-
-        public boolean isPersistent() {
-            return persistent;
-        }
-
-        public boolean isNewTimer() {
-            return newTimer;
-        }
-
         public TimerImpl build(final TimerServiceImpl timerService) {
             return new TimerImpl(this, timerService);
         }
